@@ -109,7 +109,7 @@ class PathResolver
   SLASH = '/'
   BACKSLASH = '\\'
   DOUBLE_SLASH = '//'
-  WindowsRootRx = /^(?:[a-zA-Z]:)?[\\\/]/
+  WindowsRootRx = %r(^(?:[a-zA-Z]:)?[\\/])
 
   attr_accessor :file_separator
   attr_accessor :working_dir
@@ -290,8 +290,8 @@ class PathResolver
       # ex. ./sample/path
       elsif posix_path.start_with? DOT_SLASH
         root = DOT_SLASH
-      # else ex. sample/path
       end
+      # otherwise ex. sample/path
     elsif root? posix_path
       # ex. //sample/path
       if unc? posix_path
@@ -306,8 +306,8 @@ class PathResolver
     # ex. ./sample/path
     elsif posix_path.start_with? DOT_SLASH
       root = DOT_SLASH
-    # else ex. sample/path
     end
+    # otherwise ex. sample/path
 
     path_segments = (root ? (posix_path.slice root.length, posix_path.length) : posix_path).split SLASH
     # strip out all dot entries
@@ -331,11 +331,12 @@ class PathResolver
 
   # Public: Securely resolve a system path
   #
-  # Resolve a system path from the target relative to the start path, jail path, or working
-  # directory (specified in the constructor), in that order. If a jail path is specified, enforce
-  # that the resolved path descends from the jail path. If a jail path is not provided, the resolved
-  # path may be any location on the system. If the resolved path is absolute, use it as is (unless
-  # it breaches the jail path). Expand all parent and self references in the resolved path.
+  # Resolves the target to an absolute path on the current filesystem. The target is assumed to be
+  # relative to the start path, jail path, or working directory (specified in the constructor), in
+  # that order. If a jail path is specified, the resolved path is forced to descend from the jail
+  # path. If a jail path is not provided, the resolved path may be any location on the system. If
+  # the target is an absolute path, use it as is (unless it breaches the jail path). Expands all
+  # parent and self references in the resolved path.
   #
   # target - the String target path
   # start  - the String start path from which to resolve a relative target; falls back to jail, if
@@ -347,8 +348,9 @@ class PathResolver
   #            automatically recover when an illegal path is encountered
   #          * :target_name is used in messages to refer to the path being resolved
   #
-  # returns a String path relative to the start path, if specified, and confined to the jail path,
-  # if specified. The path is posixified and all parent and self references in the path are expanded.
+  # Returns an absolute String path relative to the start path, if specified, and confined to the
+  # jail path, if specified. The path is posixified and all parent and self references in the path
+  # are expanded.
   def system_path target, start = nil, jail = nil, opts = {}
     if jail
       raise ::SecurityError, %(Jail is not an absolute path: #{jail}) unless root? jail
@@ -362,7 +364,7 @@ class PathResolver
         if jail && !(descends_from? target_path, jail)
           if opts.fetch :recover, true
             logger.warn %(#{opts[:target_name] || 'path'} is outside of jail; recovering automatically)
-            target_segments, _ = partition_path target_path
+            target_segments, = partition_path target_path
             jail_segments, jail_root = partition_path jail
             return join_path jail_segments + target_segments, jail_root
           else
@@ -371,7 +373,7 @@ class PathResolver
         end
         return target_path
       else
-        target_segments, _ = partition_path target
+        target_segments, = partition_path target
       end
     else
       target_segments = []
@@ -387,7 +389,7 @@ class PathResolver
           return expand_path start
         end
       else
-        target_segments, _ = partition_path start
+        target_segments, = partition_path start
         start = jail || @working_dir
       end
     elsif start.nil_or_empty?
@@ -419,7 +421,7 @@ class PathResolver
     if (resolved_segments = start_segments + target_segments).include? DOT_DOT
       unresolved_segments, resolved_segments = resolved_segments, []
       if jail
-        jail_segments, _ = partition_path jail unless jail_segments
+        jail_segments, = partition_path jail unless jail_segments
         warned = false
         unresolved_segments.each do |segment|
           if segment == DOT_DOT
@@ -450,7 +452,7 @@ class PathResolver
         target_path
       elsif opts.fetch :recover, true
         logger.warn %(#{opts[:target_name] || 'path'} is outside of jail; recovering automatically)
-        jail_segments, _ = partition_path jail unless jail_segments
+        jail_segments, = partition_path jail unless jail_segments
         join_path jail_segments + target_segments, jail_root
       else
         raise ::SecurityError, %(#{opts[:target_name] || 'path'} #{target} is outside of jail: #{jail} (disallowed in safe mode))
